--
-- Windrower
-- Class for all windrowers
-- Note: the animation is not perfectly synchronized, thus do not attach any gameplay relevant stuff to the animation
--
-- @author  Stefan Geiger
-- @date  11/05/08
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.

source("dataS/scripts/vehicles/specializations/WindrowAreaEvent.lua");

WindrowerMini = {};

function WindrowerMini.prerequisitesPresent(specializations)
    return true;
end;

function WindrowerMini:load(xmlFile)

    self.groundReferenceThreshold = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.groundReferenceNode#threshold"), 0.2);
    self.groundReferenceNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.groundReferenceNode#index"));
    if self.groundReferenceNode == nil then
        self.groundReferenceNode = self.components[1].node;
    end;

    local numWindrowerDropAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.windrowerDropAreas#count"), 0);
    if numWindrowerDropAreas == 0 then
        print("Warning: No drop areas specified in '"..self.configFileName.."'");
    else
        if numWindrowerDropAreas ~= 1 and numWindrowerDropAreas ~= table.getn(self.cuttingAreas) then
            print("Warning: Number of cutting areas and drop areas should be equal in '"..self.configFileName.."'");
        end;
    end;
    self.windrowerDropAreas = {}
    for i=1, numWindrowerDropAreas do
        self.windrowerDropAreas[i] = {};
        local areanamei = string.format("vehicle.windrowerDropAreas.windrowerDropArea%d", i);
        self.windrowerDropAreas[i].start = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#startIndex"));
        self.windrowerDropAreas[i].width = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#widthIndex"));
        self.windrowerDropAreas[i].height = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#heightIndex"));
    end;

    local numCuttingAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.cuttingAreas#count"), 0);
    self.accumulatedCuttingAreaValues = {};
    for i=1, numCuttingAreas do
        self.accumulatedCuttingAreaValues[i] = 0;
    end;

    self.windrowerGroundFlag = self.nextDirtyFlag;
    self.nextDirtyFlag = self.windrowerGroundFlag*2;
end;

function WindrowerMini:delete()
end;

function WindrowerMini:mouseEvent(posX, posY, isDown, isUp, button)
end;

function WindrowerMini:keyEvent(unicode, sym, modifier, isDown)
end;

function WindrowerMini:readStream(streamId, connection)
end;

function WindrowerMini:writeStream(streamId, connection)
end;

function WindrowerMini:readUpdateStream(streamId, timestamp, connection)
end;

function WindrowerMini:writeUpdateStream(streamId, connection, dirtyMask)
end;

function WindrowerMini:update(dt)
end;

function WindrowerMini:updateTick(dt)
    if self:getIsActive() then
        if self.isTurnedOn then
            if self.isServer then
                    local x,y,z = getWorldTranslation(self.groundReferenceNode);
                    local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
                    if terrainHeight+self.groundReferenceThreshold >= y then
                        local numDropAreas = table.getn(self.windrowerDropAreas);
                        local numAreas = table.getn(self.cuttingAreas);
                        if numDropAreas > 0 and numAreas > 0 then
                            local sum = 0;
                            local fruitType = FruitUtil.FRUITTYPE_GRASS;
                            local fruitTypeFix = false;
                            local foldAnimTime = self.foldAnimTime;
                            local cuttingAreasSend = {};
                            local dropAreasSend = {};
                            for i=1, numAreas do
                                local cuttingArea = self.cuttingAreas[i];
                                if self:getIsAreaActive(cuttingArea) then
                                    local x,y,z = getWorldTranslation(cuttingArea.start);
                                    local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
                                    local x2,y2,z2 = getWorldTranslation(cuttingArea.height);

                                    table.insert(cuttingAreasSend, {x,z,x1,z1,x2,z2});
                                end;
                            end;
                            for i=1, numDropAreas do
                                local dropArea = self.windrowerDropAreas[i];
                                local dx,dy,dz = getWorldTranslation(dropArea.start);
                                local dx1,dy1,dz1 = getWorldTranslation(dropArea.width);
                                local dx2,dy2,dz2 = getWorldTranslation(dropArea.height);
                                table.insert(dropAreasSend, {dx, dz, dx1, dz1, dx2, dz2});
                            end;
                            if table.getn(cuttingAreasSend) > 0 then
                                local cuttingAreasSend, dropAreasSend, fruitType, bitType = WindrowAreaEvent.runLocally(cuttingAreasSend, dropAreasSend, self.accumulatedCuttingAreaValues);
                                if table.getn(cuttingAreasSend) > 0 then
                                    g_server:broadcastEvent(WindrowAreaEvent:new(cuttingAreasSend, dropAreasSend, fruitType, bitType));
                                end;
                            end;
                        end;
                    end;
            end;
        end;
    end;
end;

function WindrowerMini:draw()
end;

function WindrowerMini:onDetach()
end;

function WindrowerMini:onLeave()
end;

function WindrowerMini:onDeactivate()
end;

function WindrowerMini:onDeactivateSounds()
end;
